/*
 * CKEditor - The text editor for Internet - http://ckeditor.com
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 */

CKEDITOR.dialog.add( 'table', function( editor )
{
	var makeElement = function( name ){ return new CKEDITOR.dom.element( name, editor.document ); },
		widthPattern = /^(\d+(?:\.\d+)?)(px|%)$/,
		heightPattern = /^(\d+(?:\.\d+)?)px$/;

	return {
		title : editor.lang.dlgTableTitle,
		minWidth : 480,
		minHeight : 290,
		onShow : function()
		{
			// Detect if there's a selected table.
			this.restoreSelection();
			var selection = editor.getSelection(),
				ranges = selection.getRanges(),
				me = this,
				selectedTable;
			if ( ranges.length > 0 )
			{
				var rangeRoot = ranges[0].getCommonAncestor( true );
				selectedTable = rangeRoot.getAscendant( 'table', true );
			}

			// Fill in relevant fields if there's a selected table.
			if ( selectedTable )
			{
				var syncAttr = function( inputName, attrName )
				{
					me.setValueOf( 'info', inputName, selectedTable.getAttribute( attrName ) );
				},
					widthMatch = widthPattern.exec( selectedTable.$.style.width ),
					heightMatch = heightPattern.exec( selectedTable.$.style.height );
				syncAttr( 'txtBorder', 'border' );
				syncAttr( 'cmbAlign', 'align' );
				syncAttr( 'txtCellPad', 'cellpadding' );
				syncAttr( 'txtCellSpace', 'cellspacing' );
				syncAttr( 'txtSummary', 'summary' );

				if ( widthMatch )
				{
					this.setValueOf( 'info', 'txtWidth', widthMatch[1] );
					this.setValueOf( 'info', 'cmbWidthType', widthMatch[2] == 'px' ? 'pixels' : 'percents' );
				}

				if ( heightMatch )
					this.setValueOf( 'info', 'txtHeight', heightMatch[1] );
			}

			// Disable the width and height fields if a table is already selected.
			// TODO

			// Put focus into the first relevant text field.
			this.getContentElement( 'info', selectedTable ? 'txtWidth' : 'txtRows' ).select();
		},
		onOk : function()
		{
			var table = this._.selectedElement || makeElement( 'table' ),
				me = this,
				caption = this.getValueOf( 'info', 'txtCaption' );

			// Insert the caption, if any.
			if ( caption != '' )
				table.append( CKEDITOR.dom.element.createFromHtml( '<caption>' 
						+ CKEDITOR.tools.htmlEncode( caption ) + '</caption>', editor.document ) );

			// Generate the rows and cols.
			if ( !this._.selectedElement )
			{
				var tbody = table.append( makeElement( 'tbody' ) ),
					rows = parseInt( this.getValueOf( 'info', 'txtRows' ), 10 ) || 0;
					cols = parseInt( this.getValueOf( 'info', 'txtCols' ), 10 ) || 0;

				for ( var i = 0 ; i < rows ; i++)
				{
					var row = tbody.append( makeElement( 'tr' ) );
					for ( var j = 0 ; j < cols ; j++ )
					{
						var cell = row.append( makeElement( 'td' ) );
						if ( !CKEDITOR.env.ie )
							cell.append( makeElement( 'br' ) );
					}
				}
			}

			// Set the basic attributes.
			var setAttr = function( inputNameOrFunction, attrName )
			{
				var value = ( typeof( inputNameOrFunction) == 'function' ) ? inputNameOrFunction()
					: me.getValueOf( 'info', inputNameOrFunction );
				if ( value != '' )
					table.setAttribute( attrName, value );
			};
			setAttr( 'txtBorder', 'border' );
			setAttr( 'cmbAlign', 'align' );
			setAttr( 'txtCellPad', 'cellpadding' );
			setAttr( 'txtCellSpace', 'cellspacing' );
			setAttr( 'txtSummary', 'summary' );
			setAttr( function()
				{
					var styles = [];
					if ( me.getValueOf( 'info', 'txtHeight' ) != '' )
						styles.push( 'height:' + me.getValueOf( 'info', 'txtHeight' ) + 'px' );
					if ( me.getValueOf( 'info', 'txtWidth' ) != '' )
					{
						var type = me.getValueOf( 'info', 'cmbWidthType' );
						styles.push( 'width:' + me.getValueOf( 'info', 'txtWidth' ) + ( type == 'pixels' ? 'px' : '%' ) );
					}
					return styles.join( ';' );
				}, 'style' );

			// Insert the table element.
			this.restoreSelection();
			editor.insertElement( table );
			this.clearSavedSelection();

			return true;
		},
		contents : [
			{
				id : 'info',
				label : editor.lang.dlgTableTitle,
				accessKey : 'I',
				elements :
				[
					{
						type : 'hbox',
						widths : [ '40%', '10%', '60%' ],
						children : 
						[
							{
								type : 'vbox',
								children : 
								[
									{
										type : 'text',
										id : 'txtRows',
										labelLayout : 'horizontal',	
										widths : [ '60%','40%' ],
										style : 'width:105px',
										'default' : 3,
										label : editor.lang.dlgTableRows,
										validate: function( ) {
											if ( this.getValue() != '' )
											{
												if ( isNaN( parseInt( this.getValue(), 10 ) ) )
												{
													this.select();
													return false;
												}
											}
											return true;
										}
									},
									{
										type : 'text',
										id : 'txtCols',
										labelLayout : 'horizontal',	
										widths : [ '60%','40%' ],
										style : 'width:105px',
										'default' : '2',
										label : editor.lang.dlgTableColumns,
										validate: function( ) {
											if ( this.getValue() != '' )
											{
												if ( isNaN( parseInt( this.getValue(), 10 ) ) )
												{
													this.select();
													return false;
												}
											}
											return true;
										}
									},
									{
										type : 'text',
										id : 'txtBorder',
										labelLayout : 'horizontal',	
										widths : [ '60%','40%' ],
										style : 'width:105px',
										'default' : '1',
										label : editor.lang.dlgTableBorder,
										validate: function( ) {
											if ( this.getValue() != '' )
											{
												if ( isNaN( parseInt( this.getValue(), 10 ) ) )
												{
													this.select();
													return false;
												}
											}
											return true;
										}
									},
									{
										id : 'cmbAlign',
										type : 'select',
										labelLayout : 'horizontal',	
										widths : [ '40%','60%' ],
										label : editor.lang.dlgTableAlign,
										items :
										[
											[ editor.lang.dlgTableAlignNotSet , ''],
											[ editor.lang.dlgTableAlignLeft , 'left'],
											[ editor.lang.dlgTableAlignCenter , 'center'],
											[ editor.lang.dlgTableAlignRight , 'right']
										]
									}
								]
							},
							{
								type : 'html',
								align : 'right',
								html : ''
							},						
							{
								type : 'vbox',
								align : 'right',
								children : 
								[			
									{
										type : 'hbox',
										align : 'center',
										widths : [ '70%', '30%' ],
										children : 
										[
											{
												type : 'text',
												id : 'txtWidth',
												labelLayout : 'horizontal',	
												widths : [ '50%','50%' ],
												label : editor.lang.dlgTableWidth,
												'default' : 200,
												validate: function( ) {
													if ( this.getValue() != '' )
													{
														if ( isNaN( parseInt( this.getValue(), 10 ) ) )
														{
															this.select();
															return false;
														}
													}
													return true;
												}
											},
											{
												id : 'cmbWidthType',
												type : 'select',
												labelLayout : 'horizontal',	
												widths : [ '0%','100%' ],
												label : '',
												'default' : 'pixels',
												items :
												[
													[ editor.lang.dlgTableWidthPx , 'pixels'],
													[ editor.lang.dlgTableWidthPc , 'percents']
												]
											}
										]
									},
									{
										type : 'hbox',
										widths : [ '70%', '30%' ],
										children : 
										[
											{
												type : 'text',
												id : 'txtHeight',
												labelLayout : 'horizontal',
												widths : [ '50%','50%' ],
												label : editor.lang.dlgTableHeight,
												validate: function( ) {
													if ( this.getValue() != '' )
													{
														if ( isNaN( parseInt( this.getValue(), 10 ) ) )
														{
															this.select();
															return false;
														}
													}
													return true;
												}
											},
											{
												type : 'html',
												html : editor.lang.dlgTableWidthPx
											}	
										]
									},
									{
										type : 'text',
										id : 'txtCellSpace',
										labelLayout : 'horizontal',	
										widths : [ '50%','50%' ],
										style : 'width:140px',
										label : editor.lang.dlgTableCellSpace,
										validate: function( ) {
											if ( this.getValue() != '' )
											{
												if ( isNaN( parseInt( this.getValue(), 10 ) ) )
												{
													this.select();
													return false;
												}
											}
											return true;
										}
									},
									{
										type : 'text',
										id : 'txtCellPad',
										labelLayout : 'horizontal',	
										widths : [ '50%','50%' ],
										style : 'width:140px',
										label : editor.lang.dlgTableCellPad,
										validate: function( ) {
											if ( this.getValue() != '' )
											{
												if ( isNaN( parseInt( this.getValue(), 10 ) ) )
												{
													this.select();
													return false;
												}
											}
											return true;
										}
									}
								]
							}
						]
					},
					{
						type : 'html',
						align : 'right',
						html : ''
					},	
					{
						id : 'txtCaption',
						type : 'text',
						label : editor.lang.dlgTableCaption,
						widths : [ '30%','70%' ],
						labelLayout : 'horizontal',	
						style : 'width:400px'
					},
					{
						id : 'txtSummary',
						type : 'text',
						labelLayout : 'horizontal',	
						label : editor.lang.dlgTableSummary,
						widths : [ '30%','70%' ],
						accessKey : 'A',
						style : 'width:400px'
					}
				]
			}
		]
	};
} );
